from Products.CMFCore.utils import getToolByName


def installPrerequisites(portal, out, prereqs=[], nofail=False):

    """
    Install all prerequisites. This should be a list of
    Plone product names, or a list of tuples, where each first element is the
    product name, and the second element is a version, or a list of possible versions.
    If nofail is set to True, the install method
    will not raise an exception on a non-installable product.
    """

    qi = getToolByName(portal, 'portal_quickinstaller')

    print >> out, "Installing prerequisites"

    if not len(prereqs):
        print >> out, "-> no prerequisites given"
        return

    all_products = qi.listInstalledProducts() + qi.listInstallableProducts()

    for prereq in prereqs:

        version = None

        if isinstance(prereq, tuple):
            version = prereq[1]
            if not isinstance(version, list):
                version = [version]
            prereq = prereq[0]            
        
        if qi.isProductInstallable(prereq):
            
            if version:
                if not qi.getProductVersion(prereq) in version:
                    if nofail:
                        print >> out, "-> Couldn't install %s, version %s" % (prereq, version)
                    else:
                        raise "Couldn't install prerequisite %s, version %s" % (prereq, version)

            try:
                print >> out, "-> Installing %s" % prereq
                qi.installProduct(prereq)
            except:
                if nofail:
                    print >> out, "-> Couldn't install %s" % prereq
                else:
                    raise "Couldn't install prerequisite %s" % prereq
        else:
            if nofail:
                print >> out, "-> Couldn't install %s" % prereq
            else:
                raise "Couldn't install prerequisite %s" % prereq
