from installPermissions import installPermissions
from installSetAddableTypesForObject import installSetAddableTypesForObject


def installStructure(portal, out, struct=[], reset_attributes=False, reset_permissions=True):
    
    """
    Create the given structure in the portal. The structure
    should be given in tuples, where each tuple contains the
    id for the object to create, the type, a map of values to
    set (done with setters) on the created object, a list of
    nested objects to create and a list of permissions to set
    on the object.
    
    Example structure:
    
    STRUCT = [('map0', 'Folder', {'title':'Map 0'}, [('submap0', 'Folder', {}, [])]),
              ('map1', 'Folder', {'title':'Map 1'}, [],
               [('View', 0, ['Manager'])]),
    ]
    
    If reset_attributes is set to true, given attributes will be set on objects
    if they already exist.
    
    If reset_permissions is set to true, permissions will be set on objects
    if they already exist.
    
    """
    
    if getattr(portal, 'portal_type', None) == 'Plone Site':
        print >> out, "Creating portal structure..."

    parent = portal
    
    for s in struct:
        
        obj = None
        new = False
        
        if len(s) < 4:
            print >> out, "-> tuple for %s doesn't have four parameters. Skip." % \
            str(s)
            continue
        
        if not hasattr(parent.aq_explicit, s[0]):
            try:
                print >> out, "-> Creating object %s" % s[0]
                parent.invokeFactory(s[1], s[0])
                new = True
            except:
                print >> out, "-> Exception in creating object of type %s" % \
                    s[1]

        try:            
            obj = getattr(parent, s[0])
        except:
            print >> out, "-> Can't find created object... %s" % s[0]

        # Only set the attributes for newly created objects.
        # keep existing values for already existing objects
        # or else someone might have changed e.g. the title, and then
        # those changes will be lost on every reinstall
        if obj:
            
            # Map keys to setters, and set
            #
            if new or reset_attributes:
                for key in s[2].keys():
                    setter = "set%s" % key[0].upper() + key[1:]
                    method = getattr(obj, setter, None)
                    if method:
                        print >> out, "-> calling %s with value %s" % (setter, s[2][key])
                        method(s[2][key])
                    else:
                        print >> out, "-> no setter found for %s" % key
    
            # Do we have permissions to set?
            #
            if len(s) > 4 and (new or reset_permissions):
                installPermissions(obj, out, s[4])

            # reindex
            obj.reindexObject()

            # And the children
            #
            installStructure(obj, out, s[3], reset_attributes=reset_attributes, reset_permissions=reset_permissions)
